package org.jboss.qa.brms.performance;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.conf.EventProcessingOption;
import org.drools.io.impl.ClassPathResource;
import org.drools.runtime.KnowledgeSessionConfiguration;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.conf.ClockTypeOption;
import org.drools.runtime.rule.WorkingMemoryEntryPoint;
import org.jboss.qa.brms.performance.domain.Account;
import org.jboss.qa.brms.performance.domain.NewCustomerEvent;
import org.jboss.qa.brms.performance.domain.Transaction;
import org.jboss.qa.brms.performance.domain.TransactionCreatedEvent;
import org.junit.Test;

import java.util.ArrayList;
import java.util.List;

public class AdvancedOperatorsSimpleTest {
    private static int LOOPS = 20;
    private static int NR_OF_FACTS = 500000;

    private KnowledgeBase kbase;

    @Test
    public void testCreateKBaseInsertFactsAndFireRules() {
        createKBase("expert-advanced-operators4-100.drl");
        for (int i = 0; i < LOOPS; i++) {
            executeOneLoop(i);
        }
    }

    private void createKBase(String drlName) {
        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
        kbuilder.add(new ClassPathResource(drlName), ResourceType.DRL);
        KnowledgeBaseConfiguration kbconf = KnowledgeBaseFactory.newKnowledgeBaseConfiguration();
        kbconf.setOption(EventProcessingOption.CLOUD);

        if (kbuilder.hasErrors()) {
            throw new RuntimeException(kbuilder.getErrors().toString());
        }

        kbase = KnowledgeBaseFactory.newKnowledgeBase(kbconf);
        kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
    }

    private void executeOneLoop(int loopNumber) {
        StatefulKnowledgeSession ksession = createStatefulKSession();
        long start = System.nanoTime();
        for (Object fact : getFacts(NR_OF_FACTS)) {
            ksession.insert(fact);
        }
        ksession.fireAllRules();
        long took = System.nanoTime() - start;
        System.out.println("Loop #" + loopNumber + " took " + took / 1000 / 1000 + "ms.");
        ksession.dispose();
        System.gc();
    }

    private StatefulKnowledgeSession createStatefulKSession() {
        KnowledgeSessionConfiguration ksconf = KnowledgeBaseFactory.newKnowledgeSessionConfiguration();
        ksconf.setOption(ClockTypeOption.get("realtime"));
        StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession(ksconf, null);
        return ksession;
    }

    private List<Object> getFacts(int nrOfFacts) {
        List<Object> facts = new ArrayList<Object>();
        int init = 1000;
        int diff = 5211;

        for (int i = 0; i < 20; i++) {
            for (int j = 0; j < nrOfFacts / 20 / 3; j++) {
                Account account = new Account();
                account.setBalance(i * 123456);
                String uuid = "totalAverageMinMaxlBalanceForCertainAccounts_accumulate_" + (init + i * diff);
                account.setUuid(uuid);
                facts.add(account);

                account = new Account();
                account.setUuid("transactionsWithCertainAccountFrom_collect_" + (init + i * diff));
                facts.add(account);
                Transaction transaction = new Transaction();
                transaction.setAccountFrom(account);
                facts.add(transaction);
                transaction = new Transaction();
                transaction.setAccountFrom(account);
                facts.add(transaction);
            }
        }
        return facts;
    }

}
