package org.jboss.qa.brms.performance;

import org.jboss.qa.brms.performance.domain.Account;
import org.jboss.qa.brms.performance.domain.Transaction;
import org.junit.Test;
import org.kie.api.KieBase;
import org.kie.api.conf.EventProcessingOption;
import org.kie.api.runtime.KieSession;
import org.kie.internal.builder.conf.PhreakOption;
import org.kie.internal.utils.KieHelper;

import java.util.ArrayList;
import java.util.List;

public class AdvancedOperatorsSimpleTest {
    private static int LOOPS = 20;
    private static int NR_OF_FACTS = 500000;

    private KieBase kieBase;

    @Test
    public void testCreateKBaseInsertFactsAndFireRules() {
        createKBase("expert-advanced-operators4-100.drl");
        for (int i = 0; i < LOOPS; i++) {
            executeOneLoop(i);
        }
    }

    private void createKBase(String drlName) {
        KieHelper kieHelper = new KieHelper();
        kieHelper.addFromClassPath("/" + drlName);
        kieBase = kieHelper.build(PhreakOption.ENABLED, EventProcessingOption.CLOUD);
    }

    private void executeOneLoop(int loopNumber) {
        KieSession kieSession = kieBase.newKieSession();
        long start = System.nanoTime();
        for (Object fact : getFacts(NR_OF_FACTS)) {
            kieSession.insert(fact);
        }
        kieSession.fireAllRules();
        long took = System.nanoTime() - start;
        System.out.println("Loop #" + loopNumber + " took " + took / 1000 / 1000 + "ms.");
        kieSession.dispose();
        System.gc();
    }

    private List<Object> getFacts(int nrOfFacts) {
        List<Object> facts = new ArrayList<Object>();
        int init = 1000;
        int diff = 5211;

        for (int i = 0; i < 20; i++) {
            for (int j = 0; j < nrOfFacts / 20 / 3; j++) {
                Account account = new Account();
                account.setBalance(i * 123456);
                String uuid = "totalAverageMinMaxlBalanceForCertainAccounts_accumulate_" + (init + i * diff);
                account.setUuid(uuid);
                facts.add(account);

                account = new Account();
                account.setUuid("transactionsWithCertainAccountFrom_collect_" + (init + i * diff));
                facts.add(account);
                Transaction transaction = new Transaction();
                transaction.setAccountFrom(account);
                facts.add(transaction);
                transaction = new Transaction();
                transaction.setAccountFrom(account);
                facts.add(transaction);
            }
        }
        return facts;
    }

}
